@@ -473,11 +473,13 @@ void charreport(labelptr curlabel)
 	 cleft -= 2;
       }
       if (strptr == NULL) break;
-      charprint(_STR, strptr, locpos);
-      cleft -= strlen(_STR);
-      strncat(_STR2, _STR, cleft);
-      strncat(_STR2, " ", --cleft);
-      if (cleft <= 0) break;
+      if (strptr->type != RETURN || strptr->data.flags == 0) {
+         charprint(_STR, strptr, locpos);
+         cleft -= strlen(_STR);
+         strncat(_STR2, _STR, cleft);
+         strncat(_STR2, " ", --cleft);
+         if (cleft <= 0) break;
+      }
    }
    W3printf("%s", _STR2);
 }
@@ -519,10 +521,11 @@ Boolean labeltext(int keypressed, char *clientdata)
    Boolean r = True, do_redraw = False;
    short tmplength, tmpheight, cfont;
    TextExtents tmpext;
+   TechPtr oldtech, newtech;
 
    curlabel = TOLABEL(EDITPART);
 
-   if (curlabel == NULL || curlabel->type != LABEL || areawin->textpos <= 0) {
+   if (curlabel == NULL || curlabel->type != LABEL) {
       Wprintf("Error:  Bad label string");
       eventmode = NORMAL_MODE;
       return FALSE;
@@ -655,6 +658,10 @@ Boolean labeltext(int keypressed, char *clientdata)
 			(strstr(curlabel->string->nextpart->data.string, "::")
 			== NULL))
 	       libobjname = techptr + 2;
+
+	    /* Save a pointer to the old technology before we overwrite the name */
+	    oldtech = GetObjectTechnology(libobj);
+
 	    strcpy(libobjname, curlabel->string->nextpart->data.string);
 
 	    /* If checkname() alters the name, it has to be copied back to */
@@ -670,6 +677,17 @@ Boolean labeltext(int keypressed, char *clientdata)
 	       redrawtext(curlabel);
 	    }
 	    AddObjectTechnology(libobj);
+
+	    /* If the technology name has changed, then both the old	*/
+	    /* technology and the new technology need to be marked as	*/
+	    /* having been modified.					*/
+
+	    newtech = GetObjectTechnology(libobj);
+
+	    if (oldtech != newtech) {
+	       if (oldtech) oldtech->flags |= (u_char)TECH_CHANGED;
+	       if (newtech) newtech->flags |= (u_char)TECH_CHANGED;
+	    }
 	 }
 
 	 /* Check if we altered a page name */
@@ -699,7 +717,12 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       setdefaultfontmarks();
       setcolormark(areawin->color);
-      if ((labelbuf = get_original_string(curlabel)) != NULL) {
+
+      if (!hasstuff) {
+	 /* Delete empty labels */
+	 standard_element_delete(NORMAL);
+      }
+      else if ((labelbuf = get_original_string(curlabel)) != NULL) {
 
 	 /* If the original label (before modification) is a pin in a	*/
 	 /* schematic/symbol with a matching symbol/schematic, and the	*/
@@ -744,7 +767,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 			areawin->topinstance);
 	 if (curpos != NULL)
-	    if (curpos->type == RETURN)
+	    if (curpos->type == RETURN || curpos->type == MARGINSTOP)
 	       break;
       }
    }
@@ -753,7 +776,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	 areawin->textpos--;
 	 curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 	 		areawin->topinstance);
-	 if (curpos->type == RETURN) {
+	 if (curpos->type == RETURN || curpos->type == MARGINSTOP) {
 	    if (areawin->textpos > 1) areawin->textpos--;
 	    break;
 	 }
@@ -816,14 +839,34 @@ Boolean labeltext(int keypressed, char *clientdata)
 
       /* erase first before redrawing unless the string is empty */
       undrawtext(curlabel);
+
+      /* Get text width first.  Don't back up over spaces;  this	*/
+      /* allows the margin width to be padded out with spaces.		*/
+
+      if (keypressed == MARGINSTOP)
+	 tmpext = ULength(curlabel, areawin->topinstance, areawin->textpos, NULL);
+
       if (locpos > 0) {
-         curpos = splitstring(areawin->textpos, &curlabel->string,
+	 if (keypressed == MARGINSTOP) {
+	    /* Move forward by any spaces; if we're at the text */
+	    /* end, move to the next text part;  otherwise,	*/
+	    /* split the string.				*/
+
+	    while (*(curpos->data.string + locpos) == ' ') locpos++;
+	    if (*(curpos->data.string + locpos) == '\0') locpos = 0;
+	 }
+	 if (locpos > 0)
+            curpos = splitstring(areawin->textpos, &curlabel->string,
 			areawin->topinstance);
 	 curpos = curpos->nextpart;
       }
       newpart = makesegment(&curlabel->string, curpos);
       newpart->type = keypressed;
       switch (keypressed) {
+	 case RETURN:
+	    // Identify this as an explicitly placed line break
+	    newpart->data.flags = 0;
+	    break;
 	 case FONT_SCALE:
 	    newpart->data.scale = *((float *)clientdata);
 	    break;
@@ -839,6 +882,17 @@ Boolean labeltext(int keypressed, char *clientdata)
 	    newpart->data.font = *((int *)clientdata);
 	    if (newpart->data.font >= fontcount) errcond = True;
 	    break;
+	 case MARGINSTOP:
+	    /* A margin of 1 or 0 is useless, so such a value	*/
+	    /* indicates to take the margin from the current	*/
+	    /* position.					*/
+
+	    if (*((int *)clientdata) <= 1)
+	       newpart->data.width = (int)tmpext.width;
+	    else
+	       newpart->data.width = *((int *)clientdata);
+	    CheckMarginStop(curlabel, areawin->topinstance, FALSE);
+	    break;
 	 case PARAM_START:
 	    newpart->data.string = (char *)malloc(1 + strlen(clientdata));
 	    strcpy(newpart->data.string, clientdata);
@@ -850,7 +904,7 @@ Boolean labeltext(int keypressed, char *clientdata)
 	          areawin->textpos++;
 	          curpos = findstringpart(areawin->textpos, &locpos, curlabel->string,
 	 		areawin->topinstance);
-	       } while (curpos->type != PARAM_END);
+	       } while (curpos && (curpos->type != PARAM_END));
 	    }
 	    break;
       }
@@ -912,6 +966,9 @@ Boolean labeltext(int keypressed, char *clientdata)
    /* Redraw the label */
 
    if (do_redraw) {
+      /* Generate automatic line breaks if there is a MARGINSTOP directive */
+      CheckMarginStop(curlabel, areawin->topinstance, TRUE);
+
       XcSetFunction(GXcopy);
       redrawtext(curlabel);
    }
@@ -1205,7 +1262,7 @@ void arcbutton(int x, int y)
    arcdefaults(*newarc, userpt.x, userpt.y);
    addcycle((genericptr *)newarc, 0, 0);
 
-   XcSetXORFg(areawin->color, BACKGROUND);
+   XcSetXORFg((*newarc)->color, BACKGROUND);
    XcSetFunction(GXxor);
    UDrawArc(*newarc, xobjs.pagelist[areawin->page]->wirewidth);
 
@@ -1274,6 +1331,7 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    calcarc(newarc);
 
+   XcSetXORFg(newarc->color, BACKGROUND);
    UDrawArc(newarc, xobjs.pagelist[areawin->page]->wirewidth);
    UDrawXLine(newpos, newarc->position);
    printpos(newpos.x, newpos.y);
@@ -1328,7 +1386,7 @@ void boxbutton(int x, int y)
    snap(x, y, &userpt);
    polydefaults(*newbox, 4, userpt.x, userpt.y);
 
-   XcSetXORFg(areawin->color, BACKGROUND);
+   XcSetXORFg((*newbox)->color, BACKGROUND);
    XcSetFunction(GXxor);
    UDrawPolygon(*newbox, xobjs.pagelist[areawin->page]->wirewidth);
 
@@ -1360,6 +1418,7 @@ void trackbox(xcWidget w, caddr_t clientdata, caddr_t calldata)
    pointptr++; pointptr->y = newpos.y; pointptr->x = newpos.x;
    pointptr++; pointptr->x = newpos.x;
    
+   XcSetXORFg(newbox->color, BACKGROUND);
    UDrawPolygon(newbox, xobjs.pagelist[areawin->page]->wirewidth);
    printpos(newpos.x, newpos.y);
 
@@ -1403,6 +1462,7 @@ void trackwire(xcWidget w, caddr_t clientdata, caddr_t calldata)
 	 manhattanize(&newpos, newwire, -1, TRUE);
       tpoint->x = newpos.x;
       tpoint->y = newpos.y;
+      XcSetXORFg(newwire->color, BACKGROUND);
       UDrawPolygon(newwire, xobjs.pagelist[areawin->page]->wirewidth);
       areawin->save.x = newpos.x;
       areawin->save.y = newpos.y;
@@ -1440,7 +1500,7 @@ void startwire(XPoint *userpt)
    pointptr->x = (pointptr + 1)->x = areawin->save.x = userpt->x;
    pointptr->y = (pointptr + 1)->y = areawin->save.y = userpt->y;
 
-   XcSetXORFg(areawin->color, BACKGROUND);
+   XcSetXORFg((*newwire)->color, BACKGROUND);
    XcSetFunction(GXxor);
    UDrawPolygon(*newwire, xobjs.pagelist[areawin->page]->wirewidth);
 
@@ -1624,6 +1684,7 @@ void trackelement(xcWidget w, caddr_t clientdata, caddr_t calldata)
    {
       easydraw(*selobj, DOFORALL);
       editpoints(SELTOGENERICPTR(selobj), deltax, deltay);
+      XcSetXORFg(topobject->plist[*selobj]->color, BACKGROUND);
       easydraw(*selobj, DOFORALL);
    }
 
