@@ -559,7 +559,6 @@ short closedistance(polyptr curpoly, XPoint *cursloc)
 
 short checkbounds()
 {
-   XPoint testpt;
    long lval;
 
    /* check window-to-user space */
@@ -579,7 +578,6 @@ short checkbounds()
    lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y -
 	areawin->pcorner.y) * areawin->vscale); 
    if (lval != (long)((short)lval)) return -1;
-   UTransformbyCTM(DCTM, &(topobject->bbox.lowerleft), &testpt, 1);
 
    lval = (long)((float)(topobject->bbox.lowerleft.x + topobject->bbox.width -
 	areawin->pcorner.x) * areawin->vscale);
@@ -1291,9 +1289,9 @@ void labelbbox(labelptr labox, XPoint *npoints, objinstptr callinst)
 
    tmpext = ULength(labox, callinst, 0, NULL);
    points[0].x = points[1].x = (labox->justify & NOTLEFT ? 
-	       (labox->justify & RIGHT ? -tmpext.width :
-		-tmpext.width / 2) : 0);
-   points[2].x = points[3].x = points[0].x + tmpext.width;
+	       (labox->justify & RIGHT ? -tmpext.maxwidth :
+		-tmpext.maxwidth / 2) : 0);
+   points[2].x = points[3].x = points[0].x + tmpext.maxwidth;
    points[0].y = points[3].y = (labox->justify & NOTBOTTOM ?
 	       (labox->justify & TOP ? -tmpext.ascent :
 		-(tmpext.ascent + tmpext.base) / 2) : -tmpext.base)
@@ -2027,7 +2025,7 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    tmpext = ULength(curlabel, areawin->topinstance, 0, NULL);
 
    points[0].x = (tmpjust & NOTLEFT ?
-        (tmpjust & RIGHT ? -tmpext.width : -tmpext.width >> 1) : 0)
+        (tmpjust & RIGHT ? -tmpext.maxwidth : -tmpext.maxwidth >> 1) : 0)
 	+ xdist;
    points[0].y = (tmpjust & NOTBOTTOM ?
         (tmpjust & TOP ? -tmpext.ascent : -(tmpext.ascent + tmpext.base) / 2)
@@ -2275,13 +2273,9 @@ void UDrawBBox()
 
 void strokepath(XPoint *pathlist, short number, short style, float width)
 {
-   char         solidpart;
-   char         dashstring[3];
    float        tmpwidth;
-   short	minwidth;
 
    tmpwidth = UTopTransScale(width);
-   minwidth = max(1, (short)tmpwidth);
 
    if (!(style & CLIPMASK) || (areawin->showclipmasks == TRUE) ||
 		(areawin->clipped < 0)) {
@@ -2305,11 +2299,19 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
          SetFillStyle(dpy, areawin->gc, FillSolid);
       }
       if (!(style & NOBORDER)) {
-         /* set up dots or dashes */
-         if (style & DASHED) solidpart = (char)(4 * minwidth);
-         else if (style & DOTTED) solidpart = (char)minwidth;
-         sprintf(dashstring, "%c%c", solidpart, (char)(4 * minwidth));
          if (style & (DASHED | DOTTED)) {
+	    /* Set up dots or dashes */
+	    char dashstring[2];
+	    /* prevent values greater than 255 from folding back into	*/
+	    /* type char.  Limit to 63 (=255/4) to keep at least the	*/
+	    /* dot/gap ratio to scale when 'gap' is at its maximum	*/
+	    /* value.							*/
+	    unsigned char dotsize = min(63, max(1, (short)tmpwidth));
+	    if (style & DASHED)
+	       dashstring[0] = 4 * dotsize;
+	    else if (style & DOTTED)
+	       dashstring[0] = dotsize;
+	    dashstring[1] = 4 * dotsize;
             SetDashes(dpy, areawin->gc, 0, dashstring, 2);
             SetLineAttributes(dpy, areawin->gc, tmpwidth, LineOnOffDash,
 		CapButt, (style & SQUARECAP) ? JoinMiter : JoinBevel);
@@ -2418,7 +2420,7 @@ void UDrawArc(arcptr thearc, float passwidth)
    UfTransformbyCTM(DCTM, thearc->points, tmppoints, thearc->number);
    strokepath(tmppoints, thearc->number, thearc->style, scaledwidth);
    if (thearc->cycle != NULL) {
-      UDrawXLine(thearc->position, thearc->position);
+      UDrawXLine(thearc->position, areawin->save);
    }
 }
 
@@ -2866,7 +2868,7 @@ void TopDoLatex()
    fprintf(f, "   \\normalsize\n");
    fprintf(f, "   \\parbox{%gin}{\n", (((float)width * psscale) / 72.0) / outscale);
    fprintf(f, "   \\includegraphics[scale=%g]{%s}\\\\\n", 1.0 / outscale,
-			filename);
+    			filename);
    fprintf(f, "   %% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
 
    UPushCTM();		/* Save current state */
