@@ -62,9 +62,7 @@ extern Tcl_Interp *xcinterp;
 /* Local Variable definitions						*/
 /*----------------------------------------------------------------------*/
 
-#ifdef TCL_WRAPPER
 u_short *fontnumbers;
-#endif
 u_char nfontnumbers;
 
 /*----------------------------------------------*/
@@ -249,9 +247,13 @@ void autoscale(int page)
    int width, height;
    polyptr framebox;
 
+   /* Check if auto-fit flag is selected */
    if (!(xobjs.pagelist[page]->pmode & 2)) return;
+   /* Ignore auto-fit flag in EPS mode */
+   if (!(xobjs.pagelist[page]->pmode & 1)) return;
+
    else if (topobject->bbox.width == 0 || topobject->bbox.height == 0) {
-      Wprintf("Cannot auto-fit empty page");
+      // Wprintf("Cannot auto-fit empty page");
       return;
    }
 
@@ -485,7 +487,7 @@ void setosize(xcWidget w, objinstptr dataptr)
    if (tmpres < 0)
       tmpres = -tmpres;
 
-   if (res == 0 || tmpres <= 0) {
+   if (res == 0 || tmpres == 0) {
       Wprintf("Illegal value");
       return;
    }
@@ -624,6 +626,14 @@ void setcolorscheme(Boolean boolvalue)
    RATSNESTCOLOR = appdata.ratsnestcolor;
    BBOXCOLOR = appdata.bboxpix;
    CLIPMASKCOLOR = appdata.clipcolor;
+
+#if !defined(TCL_WRAPPER) && !defined(DOUBLEBUFFER)
+   {
+      Arg wargs[1];
+      XtSetArg(wargs[0], XtNbackground, BACKGROUND);
+      XtSetValues(areawin->area, wargs, 1);
+   }
+#endif
 }
 
 /*----------------------------------------------------------------*/
@@ -1087,7 +1097,7 @@ void setcolor(xcWidget w, int cindex)
 			areawin->topinstance);
       nextptr = findstringpart(areawin->textpos, NULL, curlabel->string,
 			areawin->topinstance);
-      if (strptr->type == FONT_COLOR) {
+      if (strptr && strptr->type == FONT_COLOR) {
 	 undrawtext(curlabel);
 	 strptr->data.color = cindex;
 	 redrawtext(curlabel);
@@ -1269,6 +1279,7 @@ short findbestfont(short curfont, short newfont, short style, short encoding) {
    short i, newstyle, newenc;
 
    if (fontcount == 0) return -1;
+   if (curfont < 0) curfont = 0;
 
    if (newfont < 0)
       newfamily = fonts[curfont].family;
