@@ -28,7 +28,7 @@
 #define setaction(b,c)	NULL, 1, (XtCallbackProc)b, c
 #define toolaction(b,c) (XtCallbackProc)b, c
 #define noaction	NULL, 0, (XtCallbackProc)DoNothing, NULL
-#define offset(a)	XtOffset(XCWindowDataPtr, a)
+#define offset(a)	Number(XtOffset(XCWindowDataPtr, a))
 
 #endif
 
@@ -289,6 +289,10 @@ menustruct WindowMenu[] = {
 };
 
 menustruct FileMenu[] = {
+#ifdef XC_WIN32
+	{"New window", action(win32_new_window, NULL)},
+	{" ", noaction},
+#endif
 	{"Read Xcircuit File", action(getfile, Number(NORMAL))},
 	{"Import Xcircuit PS", action(getfile, Number(IMPORT))},
 	{"Import background PS", action(getfile, Number(PSBKGROUND))},
@@ -300,7 +304,7 @@ menustruct FileMenu[] = {
 	{" ", noaction},
 	{"Add To Library", action(getlib, NULL)},
 	{"Load New Library", action(getuserlib, NULL)},
-	{"Save Library", action(savelibpopup, Number(0))},
+	{"Save Library", action(savelibpopup, NULL)},
 	{" ", noaction},
         {"Clear Page", action(resetbutton, Number(0))},
 	{" ", noaction},
