@@ -110,6 +110,7 @@ extern genericptr getsubpart(pathptr, int *);
 
 /* interactive manipulation of elements */
 extern void splinebutton(int, int);
+extern void updatepath(pathptr);
 extern void trackelement(xcWidget, caddr_t, caddr_t);
 extern void arcbutton(int, int);
 extern void trackarc(xcWidget, caddr_t, caddr_t);
@@ -205,6 +206,7 @@ extern Boolean checkname(objectptr);
 extern char *checkvalidname(char *, objectptr);
 extern objectptr finddot(void);
 extern void movepoints(genericptr *, short, short);
+extern void editpoints(genericptr *, short, short);
 #ifndef TCL_WRAPPER
 extern void xlib_makeobject(xcWidget, caddr_t);
 #endif
@@ -335,12 +337,12 @@ extern void savetechnology(char *, char *);
 extern void findfonts(objectptr, short *);
 extern void savefile(short);
 extern int printRGBvalues(char *, int, const char *);
-extern char *nosprint(char *);
+extern char *nosprint(char *, int *, int *);
 extern FILE *fileopen(char *, char *, char *, int);
 extern Boolean xc_tilde_expand(char *, int);
 extern Boolean xc_variable_expand(char *, int);
 extern short writelabel(FILE *, stringpart *, short *);
-extern char *writesegment(stringpart *, float *, int *);
+extern char *writesegment(stringpart *, float *, int *, int *, int *);
 extern int writelabelsegs(FILE *, short *, stringpart *);
 extern void printobjects(FILE *, objectptr, objectptr **, short *, int);
 extern void printrefobjects(FILE *, objectptr, objectptr **, short *);
@@ -701,6 +703,8 @@ extern void resolve_devnames(objectptr);
 extern void resolve_indices(objectptr, Boolean);
 extern void clear_indices(objectptr);
 extern void unnumber(objectptr);
+extern char *parsepininfo(objinstptr, char *, int);
+extern char *defaultpininfo(objinstptr, int);
 extern char *parseinfo(objectptr, objectptr, CalllistPtr, char *, char *, Boolean,
 		Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
@@ -807,6 +811,7 @@ extern short readcommand(short, FILE *);
 
 /* from graphic.c */
 
+extern void count_graphics(objectptr, short *);
 extern void UDrawGraphic(graphicptr);
 extern Imagedata *addnewimage(char *, int, int);
 extern graphicptr new_graphic(objinstptr, char *, int, int);
@@ -908,6 +913,7 @@ extern short *recurse_select_element(short, u_char);
 extern void startselect(void);
 extern void trackselarea(void);
 extern void trackrescale(void);
+extern Boolean compareselection(selection *, selection *);
 extern pointselect *addcycle(genericptr *, short, u_char);
 extern void addanticycle(pathptr, splineptr, short);
 extern void copycycles(pointselect **, pointselect **);
@@ -960,6 +966,7 @@ extern int find_ordinal(int, labelptr, objinstptr);
 extern short UDrawChar(u_char, short, short, int, int, float);
 extern void UDrawString(labelptr, int, objinstptr);
 extern void UDrawStringNoX(labelptr, int, objinstptr);
+extern void CheckMarginStop(labelptr, objinstptr, Boolean);
 extern TextExtents ULength(labelptr, objinstptr, short, XPoint *);
 extern void undrawtext(labelptr);
 extern void redrawtext(labelptr);
