@@ -14,7 +14,7 @@
 #include <stdlib.h>
 #include <string.h>
 #ifndef XC_WIN32
-#include <termios.h>    /* termios do not exist on windows */
+#include <termios.h>
 #include <unistd.h>
 #endif
 #include <time.h>
@@ -151,6 +151,13 @@ Boolean render_client(XEvent *eventPtr)
       gs_state = GS_INIT;
    } 
    else {
+      char *atomname;
+
+      atomname = XGetAtomName(dpy, eventPtr->xclient.message_type);
+      if (atomname != NULL) {
+	 fprintf(stderr, "Received client message using atom \"%s\"\n", atomname);
+	 XFree(atomname);
+      }
       return False;
    }
    return True;
@@ -242,8 +249,8 @@ void start_gs()
 {
    int std_out[2], std_err[2], ret;
 #ifdef XC_WIN32
-   unsigned int pipe_size = 8196; /* Windows need a pipe size argument */
-   int pipe_mode = _O_BINARY;     /* Windows need a pipe mode argument */
+   unsigned int pipe_size = 8196;
+   int pipe_mode = _O_BINARY;
 #endif
 #ifdef HAVE_PUTENV
    static char env_str1[128], env_str2[64];
@@ -281,13 +288,12 @@ void start_gs()
    ret = _pipe(fgs, pipe_size, pipe_mode);
    ret = _pipe(std_out, pipe_size, pipe_mode);
 #endif
-
 #ifndef GS_DEBUG
 #ifndef XC_WIN32
    ret = pipe(std_err);
 #else
    ret = _pipe(srd_err, pipe_size, pipe_mode);
-#endif /* XC_WIN32 */
+#endif  /* XC_WIN32 */
 #endif
 
    /* We need a complicated pipe here, with input going from xcircuit	*/
