@@ -11,15 +11,19 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #include "Xw/MenuBtn.h"
 #endif
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -65,7 +69,7 @@ void callwritenet(xcWidget w, pointertype mode, caddr_t calldata)
         writenet(topobject, "spice", "spc");
 	break;
      case 1:
-        writenet(topobject, "sim", "sim");
+        writenet(topobject, "flatsim", "sim");
 	break;
      case 2:
         writenet(topobject, "pcb", "pcbnet");
@@ -75,7 +79,7 @@ void callwritenet(xcWidget w, pointertype mode, caddr_t calldata)
 	break;
      case 4:
 	/* Auto-numbering:  no output generated */
-	writenet(topobject, "idxpcb", "");
+	writenet(topobject, "indexpcb", "");
 	break;
    }
 }
@@ -99,18 +103,42 @@ objectptr NameToPageObject(char *objname, objinstptr *ret_inst, int *ret_page)
    return NULL;
 }
 
+/*--------------------------------------------------------------*/
+/* Get the canonical name of an object (the part without the	*/
+/* technology prefix, if any).					*/
+/*--------------------------------------------------------------*/
+
+char *GetCanonicalName(char *fullname)
+{
+   char *canonname = strstr(fullname, "::");
+   if (canonname == NULL) return fullname;
+   return canonname + 2;
+}
+
 /*----------------------------------------------------------------------*/
 /* Find the object with the indicated name.				*/
+/*									*/
+/* WARNING:  If no library technology is given, XCircuit will return	*/
+/* the first object with the name "objname".  If there are multiple	*/
+/* objects of the same name in different technologies, then the one	*/
+/* returned may not be the one expected!				*/
 /*----------------------------------------------------------------------*/
 
 objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
 {
    int i;
    liblistptr spec;
+   Boolean notech = FALSE;
+   char *techptr;
+
+   if (strstr(objname, "::") == NULL) notech = TRUE;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (spec = xobjs.userlibs[i].instlist; spec != NULL; spec = spec->next) {
-         if (!strcmp(objname, spec->thisinst->thisobject->name)) {
+	 techptr = spec->thisinst->thisobject->name;
+	 if (notech)
+	    techptr = GetCanonicalName(spec->thisinst->thisobject->name);
+         if (!strcmp(objname, techptr)) {
 	    if (ret_inst) *ret_inst = spec->thisinst;
 	    return spec->thisinst->thisobject;
 	 }
@@ -130,7 +158,7 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
 int checkpagename(objectptr thispageobj)
 {
    int p, thispage;
-   char *objname = thispageobj->name;
+   /* char *objname = thispageobj->name; (jdk) */
    Boolean changed;
    Boolean update = False;
    char *clnptr = NULL;
@@ -162,7 +190,7 @@ int checkpagename(objectptr thispageobj)
       for (p = 0; p < xobjs.pages; p++) {
 	 if (p == thispage) continue;
          if (xobjs.pagelist[p]->pageinst != NULL) {
-	    if (!strcmp(xobjs.pagelist[p]->pageinst->thisobject->name,
+	    if (!filecmp(xobjs.pagelist[p]->pageinst->thisobject->name,
 			thispageobj->name)) {
 	       /* append ":2" to name or update suffix to ensure uniqueness */
 	       if (clnptr == NULL)
@@ -207,7 +235,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    int depth, lbus, netid, subnetid;
    buslist *sbus;
    pushlistptr seltop, nextptr;
-   objectptr nettop, thisobj, pschem;
+   objectptr nettop, pschem; /*  thisobj, (jdk) */
    char *snew;
    stringpart *ppin;
 
@@ -259,7 +287,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
          destroynets(topobject);
-         createnets(areawin->topinstance);
+         createnets(areawin->topinstance, FALSE);
       }
       if ((netlist = is_resolved(&ggen, seltop, &nettop)) != NULL) {
          depth = pushnetwork(seltop, nettop);
@@ -339,6 +367,9 @@ Boolean setobjecttype(objectptr cschem)
    genericptr *cgen;
    labelptr clab;
 
+   /* If networks are specifically prohibited. . . */
+   if (cschem->schemtype == NONETWORK) return False;
+
    /* Apply only to schematic objects */
 
    if ((cschem->schemtype != PRIMARY) && (cschem->schemtype != SECONDARY)) {
@@ -446,14 +477,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
 
 void setsymschem()
 {
-   /* This is temporary (hopefully). . . Call the Tcl function	*/
-   /* "setsymschem" which had better be defined by the wrapper.	*/
-
-   Tcl_SavedResult state;
-
-   Tcl_SaveResult(xcinterp, &state);
-   Tcl_Eval(xcinterp, "xcircuit::setsymschem");
-   Tcl_RestoreResult(xcinterp, &state);
+   XcInternalTagCall(xcinterp, 1, "schematic");
 }
 
 #else
@@ -558,9 +582,19 @@ int findpageobj(objectptr pobj)
 /* number of recursion levels.  Presumably no circuit	*/
 /* would have more than several hundred hierarchical 	*/
 /* levels.						*/
+/*							*/
+/* If "dolinks" is TRUE, ennumerate and follow pages	*/
+/* that are descendents of the top level when the	*/
+/* calling symbol instance has a "link" parameter and	*/
+/* that parameter points to the same filename as the	*/
+/* page.  This indicates a multiple-file project; the	*/
+/* pages keep separate records of changes, and are	*/
+/* saved independently, and loaded through the link	*/
+/* dependency method.					*/
 /*------------------------------------------------------*/
 
-int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
+int findsubschems(int toppage, objectptr cschem, int level, short *pagelist,
+	Boolean dolinks)
 {
    genericptr *cgen;
 
@@ -568,12 +602,32 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
       if (IS_OBJINST(*cgen)) {
-	 objectptr cobj = TOOBJINST(cgen)->thisobject;
+	 objinstptr cinst = TOOBJINST(cgen);
+	 objectptr cobj = cinst->thisobject;
 
 	 if (cobj->symschem != NULL) {
 	    int pageno = findpageobj(cobj->symschem);
-	    if ((pageno >= 0) && (pageno < xobjs.pages))
-		pagelist[pageno]++;
+
+	    if ((pageno >= 0) && (pageno < xobjs.pages)) {
+
+	       /* Look for a "link" parameter */
+	       if (dolinks == FALSE) {
+		  oparamptr ops;
+	          ops = find_param(cinst, "link");
+	          if ((ops != NULL) && (ops->type == XC_STRING)) {
+	             char *filename = textprint(ops->parameter.string, cinst);
+		     if (!strcmp(filename, "%n") || !strcmp(filename, "%N") ||
+			  !strcmp(filename, xobjs.pagelist[pageno]->filename)) {
+		        free(filename);
+		        continue;
+		     }
+	             free(filename);
+		  }
+	       }
+
+	       /* Add this page to the list */
+	       pagelist[pageno]++;
+	    }
 
 	    /* A symbol on its own schematic page is allowed for clarity */
 	    /* of the schematic, but this cannot be a functional part of */
@@ -581,13 +635,13 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 	    
 	    if (cobj->symschem != cschem) {
 	       if (findsubschems(toppage, cobj->symschem,
-				level + 1, pagelist) == -1)
+				level + 1, pagelist, dolinks) == -1)
 		  return -1;
 	    }
 	 }
 	 else if (cobj->schemtype != FUNDAMENTAL && cobj->schemtype != TRIVIAL) {
 	    /* Check symbols acting as their own schematics */
-	    if (findsubschems(toppage, cobj, level + 1, pagelist) == -1)
+	    if (findsubschems(toppage, cobj, level + 1, pagelist, dolinks) == -1)
 	       return -1;
 	 }
       }
@@ -595,15 +649,16 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
    return 0;
 }
 
-/*-------------------------------------------------------*/
-/* Recursively find all sub-circuits associated with the */
-/* top-level circuit and set their filenames to be the	 */
-/* same.						 */
-/*-------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Recursively find all sub-circuits associated with	*/
+/* the top-level circuit.  For each schematic that does	*/
+/* not have a valid filename, set the filename equal to	*/
+/* the filename of the "toppage" schematic.		*/
+/*------------------------------------------------------*/
 
 void collectsubschems(int toppage)
 {
-   int loctop, level = 0;
+   int loctop;
    objectptr cschem;
    short *pagelist, pageno;
    Pagedata *curpage;
@@ -624,20 +679,52 @@ void collectsubschems(int toppage)
    for (pageno = 0; pageno < xobjs.pages; pageno++)
       pagelist[pageno] = 0;
 
-   findsubschems(loctop, cschem, 0, pagelist);
+   findsubschems(loctop, cschem, 0, pagelist, FALSE);
 
    for (pageno = 0; pageno < xobjs.pages; pageno++) {
       if (pageno == loctop) continue;
       if (pagelist[pageno] > 0) {
-	  if (xobjs.pagelist[pageno]->filename != NULL)
-	     free(xobjs.pagelist[pageno]->filename);
-	  xobjs.pagelist[pageno]->filename =
-	     strdup(xobjs.pagelist[loctop]->filename);
+	 if (xobjs.pagelist[pageno]->filename != NULL)
+	    free(xobjs.pagelist[pageno]->filename);
+	 xobjs.pagelist[pageno]->filename =
+	    strdup(xobjs.pagelist[loctop]->filename);
       }
    } 
    free((char *)pagelist);
 }
 
+/*------------------------------------------------------*/
+/* compare_qualified(str1, str2) ---			*/
+/*							*/
+/* 	String comparison for technology-qualified	*/
+/*	names.  str2" must be a fully-qualified name.	*/
+/*	"str1" may be qualified or unqualified.  If	*/
+/*	unqualified, compare_qualified will return TRUE	*/
+/*	for any "str2" that matches the name part of	*/
+/*	the technology::name format.			*/
+/*							*/
+/*	Return value:  TRUE if match, FALSE if not.	*/
+/*------------------------------------------------------*/
+
+Boolean compare_qualified(char *str1, char *str2)
+{
+   char *sptr1, *sptr2;
+   Boolean qual1, qual2;
+
+   sptr2 = strstr(str2, "::");
+   qual2 = (sptr2 == NULL) ? FALSE : TRUE;
+
+   if (!qual2) return (!strcmp(str1, str2));
+
+   sptr1 = strstr(str1, "::");
+   qual1 = (sptr1 == NULL) ? FALSE : TRUE;
+
+   if (qual1) return (!strcmp(str1, str2));
+
+   sptr2 += 2;
+   return (!strcmp(str1, sptr2));
+}
+
 /*-------------------------------------------------------*/
 /* Check if top-level page is the same name as a library */
 /* object; if so, connect it.				 */
@@ -659,7 +746,7 @@ int checkschem(objectptr thisobj, char *cname)
       for (j = 0; j < xobjs.userlibs[i].number; j++) {
 	 tlib = xobjs.userlibs[i].library + j;
 
-         if (!strcmp(cname, (*tlib)->name)) {
+         if (compare_qualified(cname, (*tlib)->name)) {
 	    thisobj->symschem = (*tlib);
 	    thisobj->schemtype = PRIMARY;
 	    (*tlib)->symschem = thisobj;
@@ -686,7 +773,7 @@ int checksym(objectptr symobj, char *cname)
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
       if (xobjs.pagelist[cpage]->pageinst != NULL) {
          checkpage = xobjs.pagelist[cpage]->pageinst->thisobject;
-         if (!strcmp(checkpage->name, cname)) {
+         if (compare_qualified(cname, checkpage->name)) {
 	    symobj->symschem = checkpage;
 	    symobj->schemtype = SYMBOL;
 	    checkpage->symschem = symobj;
@@ -701,15 +788,19 @@ int checksym(objectptr symobj, char *cname)
 /*----------------------------------------------------------------------*/
 /* Find location of corresponding pin in symbol/schematic and change it */
 /* to the text of the indicated label.					*/
+/*									*/
+/* Return the number of other pins changed.  zero indicates that no	*/
+/* corresponding pins were found, and therefore nothing was changed.	*/
 /*----------------------------------------------------------------------*/
 
-void changeotherpins(labelptr newlabel, stringpart *oldstring)
+int changeotherpins(labelptr newlabel, stringpart *oldstring)
 {
    objectptr other = topobject->symschem;
    genericptr *tgen;
    labelptr tlab;
+   int rval = 0;
 
-   if (other == NULL) return;
+   if (other == NULL) return rval;
 
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
       if (IS_LABEL(*tgen)) {
@@ -719,10 +810,12 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
 	    if (newlabel != NULL) {
 	       free(tlab->string);
 	       tlab->string = stringcopy(newlabel->string);
+	       rval++;
 	    }
 	 }
       }
    }
+   return rval;
 }
 
 /*----------------------------------------------------------------------*/
@@ -731,7 +824,7 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
 
 void xlib_swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 {
-   swapschem((int)mode, -1);
+   swapschem((int)mode, -1, NULL);
 }
 
 /*----------------------------------------------------------------------*/
@@ -741,7 +834,7 @@ void xlib_swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 /*  happens.								*/
 /*----------------------------------------------------------------------*/
 
-void swapschem(int allow_create, int libnum)
+void swapschem(int allow_create, int libnum, char *fullname)
 {
    objectptr savepage = topobject;
    labelptr  *pinlab;
@@ -749,16 +842,15 @@ void swapschem(int allow_create, int libnum)
    Boolean lflag;
    pushlistptr stacktop;
    short loclibnum = libnum;
+   char *canonname;
 
    if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
-   if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
-      standard_element_delete(NORMAL);
-
    /* Create symbol or schematic, if allowed by allow_create */
 
    if ((topobject->symschem == NULL) && (allow_create != 0)
 		&& (topobject->schemtype != SECONDARY)) {
+
       if (topobject->schemtype != PRIMARY) {
 	 int tpage;
 
@@ -770,7 +862,7 @@ void swapschem(int allow_create, int libnum)
 	 /* Push the current instance onto the push stack */
 	 /* Change the page without destroying the pushlist */
 
-	 push_stack(&areawin->stack, areawin->topinstance);
+	 push_stack(&areawin->stack, areawin->topinstance, NULL);
 	 stacktop = areawin->stack;
 	 areawin->stack = NULL;
          changepage(tpage);
@@ -793,12 +885,15 @@ void swapschem(int allow_create, int libnum)
 
 	 incr_changes(*newobject);
 
+	 if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
+	    standard_element_delete(ERASE);
+
 	 /* Generate a library instance for this object and set the */
 	 /* top instance to point to it.			    */
 
 	 topobject->viewscale = areawin->vscale;
 	 topobject->pcorner = areawin->pcorner;
-	 push_stack(&areawin->stack, areawin->topinstance);
+	 push_stack(&areawin->stack, areawin->topinstance, NULL);
 	 areawin->topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
@@ -810,9 +905,21 @@ void swapschem(int allow_create, int libnum)
       savepage->symschem = topobject;
       topobject->symschem = savepage;
 
-      /* make the name of the new object equal to that of the old */
+      /* Make the name of the new object equal to that of the old, */
+      /* except that symbols get the full name while schematics	   */
+      /* get the canonical name (without the technology prefix)	   */
 
-      strcpy(topobject->name, savepage->name);
+      if (fullname == NULL)
+	 canonname = GetCanonicalName(savepage->name);
+      else {
+         canonname = strstr(fullname, "::");
+	 if ((canonname == NULL) || (topobject->schemtype != PRIMARY))
+	    canonname = fullname;
+	 else
+	    canonname += 2;
+      }
+      strcpy(topobject->name, canonname);
+      checkname(topobject);
 
       /* copy all pin labels into the new object */
 
@@ -847,7 +954,6 @@ void swapschem(int allow_create, int libnum)
 	       (*pinlab)->passed = NULL;
 	       u2u_snap(&((*pinlab)->position));
 	       (*pinlab)->string = stringcopy(lpin->string);
-	       topobject->parts++;
 	       incr_changes(topobject);
 	    }
          }
@@ -903,9 +1009,12 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
 	       
+	 if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
+	    delete_for_xfer(NORMAL, areawin->selectlist, areawin->selects);
+
 	 topobject->viewscale = areawin->vscale;
 	 topobject->pcorner = areawin->pcorner;
-	 push_stack(&areawin->stack, areawin->topinstance);
+	 push_stack(&areawin->stack, areawin->topinstance, NULL);
 	 areawin->topinstance = syminst;
       }
    }
@@ -930,9 +1039,7 @@ void makesymbol(xcWidget w, caddr_t calldata)
 {
    /* copy name from popup prompt buffer and check */
 
-   strcpy(topobject->name, _STR2);
-   checkname(topobject);
-   swapschem(1, -1);
+   swapschem(1, -1, _STR2);
 }
 
 /*----------------------------------------------------------------------*/
@@ -957,7 +1064,7 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
       popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, NULL);
    }
    else
-      swapschem(1, -1);
+      swapschem(1, -1, NULL);
 }
 
 #endif /* !TCL_WRAPPER */
@@ -1032,7 +1139,8 @@ Boolean schemassoc(objectptr schemobj, objectptr symbolobj)
 	 symbolobj->schemtype = SYMBOL;
 
       /* Schematic takes the name of its associated symbol, by default */
-      strcpy(schemobj->name, symbolobj->name);
+      /* Don't copy any technology prefix. */
+      strcpy(schemobj->name, GetCanonicalName(symbolobj->name));
 
       /* Ensure that schematic (page) name is unique */
       while (checkpagename(schemobj) < 0);
