@@ -1028,7 +1028,7 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 
 	   tmpext = ULength(slab, areawin->topinstance, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
-			tmpext.width : tmpext.width >> 1) : 0);
+			tmpext.maxwidth : tmpext.maxwidth >> 1) : 0);
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
 			tmpext.ascent : (tmpext.ascent + tmpext.base) >> 1)
 			: tmpext.base);
@@ -1043,7 +1043,7 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 	      if (strptr->type == RETURN) tmpl2 = 0;
 	      tmpl1 = tmpl2;
 	      tmpext = ULength(slab, areawin->topinstance, j + 1, NULL);
-	      tmpl2 = tmpext.width;
+	      tmpl2 = tmpext.maxwidth;
 	      isect = test_insideness(((tmpl1 + tmpl2) >> 1) - adj.x,
 			(tmpext.base + (BASELINE >> 1)) - adj.y, newboxpts);
 
@@ -1066,6 +1066,19 @@ Boolean selectarea(objectptr selobj, XPoint *boxpts, short level)
 	      }
 	   }
 	   if (state == 1) areawin->textpos = j;   /* selection goes to end of string */
+
+	   /* If a label happens to be empty (can happen in the case of */
+	   /* a label with parameters that are all empty strings), then */
+	   /* check if the bounding box surrounds the label position.   */
+
+	   else if (tmpext.width == 0) {
+	      isect = test_insideness(0, 0, newboxpts);
+	      if (isect) {
+		 selected = True;
+		 areawin->textend = 1;
+	      }
+	   }
+
 	   } break;
 	   
 	case(PATH):
@@ -1373,11 +1386,13 @@ void startselect()
    UDrawBox(areawin->origin, areawin->save);
 
 #ifdef TCL_WRAPPER
-   Tk_CreateEventHandler(areawin->area, ButtonMotionMask,
-		(Tk_EventProc *)xctk_drag, NULL);
+   Tk_CreateEventHandler(areawin->area, ButtonMotionMask |
+		PointerMotionMask, (Tk_EventProc *)xctk_drag,
+		NULL);
 #else
-   xcAddEventHandler(areawin->area, ButtonMotionMask, False,
-		(xcEventHandler)xlib_drag, NULL);
+   xcAddEventHandler(areawin->area, ButtonMotionMask |
+		PointerMotionMask, False, (xcEventHandler)xlib_drag,
+		NULL);
 #endif
 
 }
