@@ -260,7 +260,6 @@ void SVGCreateImages(int page)
        _spawnl(_P_WAIT, GM_EXEC, GM_EXEC, "convert", fname, outname, NULL);
        _unlink(fname);
 #endif
-
        Fprintf(stdout, "Generated standalone PNG image file %s\n", outname);
     }
     free(glist);
@@ -635,7 +634,7 @@ void SVGDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    tmpext = ULength(drawlabel, localinst, 0, NULL);
 
    newpoint.x = (tmpjust & NOTLEFT ?
-       (tmpjust & RIGHT ? -tmpext.width : -tmpext.width >> 1) : 0);
+       (tmpjust & RIGHT ? -tmpext.maxwidth : -tmpext.maxwidth >> 1) : 0);
    newpoint.y = (tmpjust & NOTBOTTOM ?
        (tmpjust & TOP ? -tmpext.ascent : -(tmpext.ascent + tmpext.base) >> 1)
 		: -tmpext.base);
