@@ -747,8 +747,7 @@ short undo_one_action()
 	    break;
 	 }
 	 else {
-	    /* XXX egen is uninitialized! */
-	    delete_one_element(thisrecord->thisinst, egen);
+	    delete_one_element(thisrecord->thisinst, (genericptr)thisinst);
 	 }
 
 	 /* Put back all the parts */
@@ -848,9 +847,10 @@ short undo_one_action()
 	    case OBJINST: {
 	       XPoint tmppt;
 	       objinstptr einst = (objinstptr)(erec->element);
+	       // Swap instance and saved positions.
 	       tmppt = einst->position;
 	       einst->position = erec->save.instpos;
-	       erec->save.instpos = ((objinstptr)egen)->position;
+	       erec->save.instpos = tmppt;
 	       drawarea(areawin->area, NULL, NULL);
 	    }  break;
 	    case POLYGON: {
@@ -1113,9 +1113,14 @@ void undo_finish_series()
 
 void undo_action()
 {
-   short idx = undo_one_action(False);
+   short idx;
+
+   // Cannot undo while in the middle of an undo series.  Failsafe.
+   if (undo_collect != (u_char)0) return;
+
+   idx = undo_one_action();
    while (xobjs.undostack && xobjs.undostack->idx == idx)
-      undo_one_action(False);
+      undo_one_action();
 }
 
 /*----------------------------------------------------------------------*/
@@ -1268,9 +1273,10 @@ short redo_one_action()
 	    case OBJINST: {
 	       XPoint tmppt;
 	       objinstptr einst = (objinstptr)(erec->element);
+	       // Swap instance position and saved position
 	       tmppt = einst->position;
 	       einst->position = erec->save.instpos;
-	       erec->save.instpos = ((objinstptr)egen)->position;
+	       erec->save.instpos = tmppt;
 	       drawarea(areawin->area, NULL, NULL);
 	    }  break;
 	    case ARC: {
@@ -1506,7 +1512,12 @@ short redo_one_action()
 
 void redo_action()
 {
-   short idx = redo_one_action();
+   short idx;
+
+   // Cannot redo while in the middle of an undo series.  Failsafe.
+   if (undo_collect != (u_char)0) return;
+
+   idx = redo_one_action();
    while (xobjs.redostack && xobjs.redostack->idx == idx)
       redo_one_action();
 }
