@@ -114,9 +114,7 @@ xcIntervalId printtime_id;
 short beeper;
 short fontcount;
 fontinfo *fonts;
-#ifdef TCL_WRAPPER
 short	 popups;      /* total number of popup widgets on the screen */
-#endif
 
 /*----------------------------------------------------------------------*/
 /* Externally defined variables						*/
@@ -333,7 +331,7 @@ u_short countchanges(char **promptstr)
 
    for (ns = xobjs.technologies; ns != NULL; ns = ns->next) {
       tech_set_changes(ns);
-      if ((ns->flags & LIBRARY_CHANGED) != 0) {
+      if ((ns->flags & TECH_CHANGED) != 0) {
 	 changes++;
 	 if (promptstr != NULL) {
 	    fname = ns->filename;
@@ -1128,12 +1126,13 @@ void pre_initialize()
    for (page = 0; page < PAGES; page++) {
       xobjs.pagelist[page] = (Pagedata *) malloc(sizeof(Pagedata));
       xobjs.pagelist[page]->pageinst = NULL;
+      xobjs.pagelist[page]->filename = NULL;
    }
    /* Set values for the first page */
    xobjs.pagelist[0]->wirewidth = 2.0;
    xobjs.pagelist[0]->outscale = 1.0;
    xobjs.pagelist[0]->background.name = (char *)NULL;
-   xobjs.pagelist[0]->pmode = 0;
+   xobjs.pagelist[0]->pmode = 2;	/* set auto-fit ON by default */
    xobjs.pagelist[0]->orient = 0;
    xobjs.pagelist[0]->gridspace = DEFAULTGRIDSPACE;
    xobjs.pagelist[0]->snapspace = DEFAULTSNAPSPACE;
