@@ -8,11 +8,11 @@
 /*----------------------------------------------------------------------*/
 
 #ifndef HAVE_U_CHAR
-typedef unsigned char       u_char;
-typedef unsigned short      u_short;
-typedef unsigned int        u_int;
-typedef unsigned long       u_long;
-typedef unsigned long long  u_long_long;
+typedef unsigned char		u_char;
+typedef unsigned short		u_short;
+typedef unsigned int		u_int;
+typedef unsigned long		u_long;
+typedef unsigned long long	u_long;
 #endif
 
 #ifdef XC_WIN32
@@ -79,6 +79,7 @@ typedef unsigned long long  u_long_long;
 /*----------------------------------------------------------------------*/
 /* Deal with 32/64 bit processors based on autoconf results.		*/
 /*----------------------------------------------------------------------*/
+
 #ifndef SIZEOF_VOID_P
 #error "SIZEOF_VOID_P undefined!"
 #endif
@@ -265,7 +266,7 @@ typedef struct {
 } XlPoint;
 
 typedef struct {
-   short width, ascent, descent, base;
+   short width, maxwidth, ascent, descent, base;
 } TextExtents;
 
 /*----------------------------------------------------------------------*/
@@ -320,6 +321,7 @@ typedef struct {
 
 /*----------------------------------------------------------------------*/
 /* Event mode definitions (state of drawing area)			*/
+/* NOTE:  changes to this list must be reflected in xctcl_eventmode().	*/
 /*----------------------------------------------------------------------*/
 
 enum editmode {
@@ -398,22 +400,23 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define FONT_NAME       13  /* inline font designator; data = font name */
 #define FONT_SCALE	14  /* font scale change; data = scale		*/
 #define FONT_COLOR	15  /* font color change; data = color		*/
-#define KERN		16  /* set new kern values; data = kern x, y	*/
-#define PARAM_START	17  /* bounds a parameter; data = param key	*/
-#define PARAM_END	18  /* bounds a parameter; no data 		*/
+#define MARGINSTOP	16  /* declare a width limit for the text	*/
+#define KERN		17  /* set new kern values; data = kern x, y	*/
+#define PARAM_START	18  /* bounds a parameter; data = param key	*/
+#define PARAM_END	19  /* bounds a parameter; no data 		*/
 
 /* Actions translated to keystates (numbering continues from above) */
 
-#define TEXT_RETURN	19
-#define TEXT_HOME	20
-#define TEXT_END	21
-#define TEXT_SPLIT	22
-#define TEXT_DOWN	23
-#define TEXT_UP		24
-#define TEXT_LEFT	25
-#define TEXT_RIGHT	26
-#define TEXT_DELETE	27
-#define TEXT_DEL_PARAM	28
+#define TEXT_RETURN	20
+#define TEXT_HOME	21
+#define TEXT_END	22
+#define TEXT_SPLIT	23
+#define TEXT_DOWN	24
+#define TEXT_UP		25
+#define TEXT_LEFT	26
+#define TEXT_RIGHT	27
+#define TEXT_DELETE	28
+#define TEXT_DEL_PARAM	29
 
 #define SPECIAL		63  /* used only when called from menu		*/
 #define NULL_TYPE	255 /* used as a placeholder			*/
@@ -620,6 +623,8 @@ typedef struct _stringpart {
      u_char	*string;
      int	color;
      int	font;
+     int	width;
+     int	flags;
      float	scale;
      short	kern[2];
    }		data;
@@ -1329,7 +1334,7 @@ enum {
    XCF_SPICE /* 104 */,		XCF_PCB /* 105 */,
    XCF_SPICEflat /* 106 */,	XCF_Rescale /* 107 */,		
    XCF_Reorder /* 108 */,	XCF_Color /* 109 */,
-   XCF_Text_Delete_Param /* 110 */,
+   XCF_Margin_Stop /* 110 */,	XCF_Text_Delete_Param /* 111 */,
    NUM_FUNCTIONS
 };
 
@@ -1482,11 +1487,12 @@ typedef struct _Technology {
 
 /* Known flags for library pages */
 
-#define LIBRARY_CHANGED		0x01
-#define LIBRARY_READONLY	0x02
-#define LIBRARY_REPLACE		0x04	/* Replace instances when reading */
-#define LIBRARY_REPLACE_TEMP	0x08	/* Temporary store */
-#define LIBRARY_USED		0x10	/* Temporary marker flag */
+#define TECH_CHANGED		0x01
+#define TECH_READONLY		0x02	/* Technology file not writable */
+#define TECH_IMPORTED		0x04	/* Loaded only part of file */
+#define TECH_REPLACE		0x08	/* Replace instances when reading */
+#define TECH_REPLACE_TEMP	0x10	/* Temporary store */
+#define TECH_USED		0x20	/* Temporary marker flag */
 
 /*----------------------------------------------------------------------*/
 /* A convenient structure for holding all the object lists		*/
