@@ -1395,8 +1395,35 @@ LRESULT CALLBACK XcTopWindowProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 			return 0;
 		case WM_ACTIVATE:
 			if (wParam != 0) {
+				XCWindowData *w;
+
+				for (w=xobjs.windowlist; w!=NULL; w=w->next)
+					if (GetAncestor(w->window, GA_ROOT) == hwnd)
+						break;
+				if (w != NULL && areawin != w) {
+					areawin = w;
+					top = get_widget(GetAncestor(areawin->window, GA_ROOT));
+					topwin = top->handle;
+					toolBar = get_widget(GetDlgItem(topwin, TOOLBAR_ID));
+					statusBar = GetDlgItem(topwin, STATUSBAR_ID);
+					{
+						MENUITEMINFO mi_info;
+						HMENU hMenu = GetMenu(hwnd);
+						int i;
+
+						ZeroMemory(&mi_info, sizeof(mi_info));
+						mi_info.cbSize = sizeof(mi_info);
+						for (i=0; i<MaxMenuWidgets; i++) {
+							mi_info.fMask = MIIM_DATA;
+							mi_info.dwItemData = (LONG)NULL;
+							if (GetMenuItemInfo(hMenu, 100+i, FALSE, &mi_info))
+								menuwidgets[i] = (Widget)mi_info.dwItemData;
+							else
+								printf("menu not found: %d\n", i);
+						}
+					}
+				}
 				SetFocus(areawin->window);
-				areawin = 
 			}
 			return 0;
 		case WM_SIZE:
@@ -1822,7 +1849,7 @@ static MenuItem insert_win_menuitem(Menu menu, MENUITEMINFO* mi_info, menustruct
 	} else if (!(mi_info->fMask & MIIM_FTYPE && mi_info->fType == MFT_SEPARATOR)){
 	}
 	if (ms != NULL) {
-		if (ms->func != NULL)
+		if (ms->func != NULL && ms->func != DoNothing)
 			WinAddCallback((Widget)item, XtNselect, ms->func, ms->passeddata);
 		item->name = strdup(ms->name);
 	}
@@ -1902,6 +1929,36 @@ static char* color_string(int c)
 	return buf;
 }
 
+static void makesubmenu(Menu menu, menustruct *items, int n);
+static void make_color_submenu(Menu menu, menustruct *items, int n)
+{
+	if (dpy == NULL)
+		makesubmenu(menu, items, n);
+	else {
+		int i;
+		MENUITEMINFO mi_info;
+		MenuItem item;
+
+		makesubmenu(menu, items, 3);
+		menuID += (n-3);
+		for (i=0; i<number_colors; i++) {
+			int color = colorlist[i].color.pixel;
+			HBITMAP hBitmap = create_color_icon(color, MENUICON_WIDTH, MENUICON_HEIGHT);
+			ZeroMemory(&mi_info, sizeof(mi_info));
+			mi_info.cbSize = sizeof(mi_info);
+			mi_info.fMask = MIIM_BITMAP | MIIM_STRING;
+			mi_info.hbmpItem = hBitmap;
+			mi_info.dwTypeData = color_string(color);
+			mi_info.dwItemData = color;
+		
+			item = insert_win_menuitem(menu, &mi_info, NULL, -1);
+			item->name = strdup(mi_info.dwTypeData);
+			WinAddCallback((Widget)item, XtNselect, (XtCallbackProc)setcolor, NULL);
+		}
+	}
+		
+}
+
 static void makesubmenu(Menu menu, menustruct *items, int n)
 {
 	int i;
@@ -1944,13 +2001,16 @@ static void makesubmenu(Menu menu, menustruct *items, int n)
 			}
 		} else {
 			Menu popup = create_win_menu(CreatePopupMenu());
-			makesubmenu(popup, items[i].submenu, items[i].size);
+			if (strncmp(items[i].name, "Color", 5) == 0)
+				make_color_submenu(popup, items[i].submenu, items[i].size);
+			else
+				makesubmenu(popup, items[i].submenu, items[i].size);
 			mi_info.fMask = MIIM_SUBMENU | MIIM_STRING;
 			mi_info.hSubMenu = popup->handle;
 			mi_info.dwTypeData = items[i].name;
 		}
 		mitem = insert_win_menuitem(menu, &mi_info, &items[i], -1);
-		if (is_color != -1)
+		if (is_color != -1 && dpy == NULL)
 			addtocolorlist((Widget)mitem, is_color);
 	}
 }
@@ -1961,6 +2021,7 @@ static HMENU create_menus()
 	MENUITEMINFO mi_info;
 	int i;
 
+	menuID = 100;
 	menu = create_win_menu(CreateMenu());
 	for (i=0; i<maxbuttons; i++) {
 		ZeroMemory(&mi_info, sizeof(mi_info));
@@ -2188,55 +2249,57 @@ XCWindowData* GUI_init(int argc, char *argv[])
 			COM_register(0);
 #endif
 
-	register_win32_classes();
+	if (dpy == NULL) {
+		register_win32_classes();
 
-	appdata.globalcolor = WinNamedColor("Orange2");
-	appdata.localcolor = WinNamedColor("Red");
-	appdata.infocolor = WinNamedColor("SeaGreen");
-	appdata.ratsnestcolor = WinNamedColor("Tan4");
-	appdata.bboxpix = WinNamedColor("greenyellow");
-	appdata.fg = WinNamedColor("Black");
-	appdata.bg = WinNamedColor("White");
-	appdata.gridpix = WinNamedColor("Gray95");
-	appdata.snappix = WinNamedColor("Red");
-	appdata.selectpix = WinNamedColor("Gold3");
-	appdata.querypix = WinNamedColor("Turquoise");
-	appdata.filterpix = WinNamedColor("SteelBlue3");
-	appdata.axespix = WinNamedColor("Antique White");
-	appdata.buttonpix = WinNamedColor("Gray85");
-	appdata.auxpix = WinNamedColor("Green3");
-	appdata.barpix = WinNamedColor("Tan");
-	appdata.parampix = WinNamedColor("Plum3");
-	appdata.fg2 = WinNamedColor("White");
-	appdata.bg2 = WinNamedColor("DarkSlateGray");
-	appdata.gridpix2 = WinNamedColor("Gray40");
-	appdata.snappix2 = WinNamedColor("Red");
-	appdata.selectpix2 = WinNamedColor("Gold");
-	appdata.querypix2 = WinNamedColor("Turquoise");
-	appdata.filterpix2 = WinNamedColor("SteelBlue1");
-	appdata.axespix2 = WinNamedColor("NavajoWhite4");
-	appdata.buttonpix2 = WinNamedColor("Gray50");
-	appdata.auxpix2 = WinNamedColor("Green");
-	appdata.barpix2 = WinNamedColor("Tan");
-	appdata.parampix2 = WinNamedColor("Plum3");
-	appdata.width = 950;
-	appdata.height = 760;
-	appdata.timeout = 10;
-	appdata.xcfont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
-	appdata.helpfont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
-	appdata.filefont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
-	appdata.textfont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
-	appdata.titlefont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
+		appdata.globalcolor = WinNamedColor("Orange2");
+		appdata.localcolor = WinNamedColor("Red");
+		appdata.infocolor = WinNamedColor("SeaGreen");
+		appdata.ratsnestcolor = WinNamedColor("Tan4");
+		appdata.bboxpix = WinNamedColor("greenyellow");
+		appdata.clipcolor = WinNamedColor("powderblue");
+		appdata.fg = WinNamedColor("Black");
+		appdata.bg = WinNamedColor("White");
+		appdata.gridpix = WinNamedColor("Gray95");
+		appdata.snappix = WinNamedColor("Red");
+		appdata.selectpix = WinNamedColor("Gold3");
+		appdata.querypix = WinNamedColor("Turquoise");
+		appdata.filterpix = WinNamedColor("SteelBlue3");
+		appdata.axespix = WinNamedColor("Antique White");
+		appdata.buttonpix = WinNamedColor("Gray85");
+		appdata.auxpix = WinNamedColor("Green3");
+		appdata.barpix = WinNamedColor("Tan");
+		appdata.parampix = WinNamedColor("Plum3");
+		appdata.fg2 = WinNamedColor("White");
+		appdata.bg2 = WinNamedColor("DarkSlateGray");
+		appdata.gridpix2 = WinNamedColor("Gray40");
+		appdata.snappix2 = WinNamedColor("Red");
+		appdata.selectpix2 = WinNamedColor("Gold");
+		appdata.querypix2 = WinNamedColor("Turquoise");
+		appdata.filterpix2 = WinNamedColor("SteelBlue1");
+		appdata.axespix2 = WinNamedColor("NavajoWhite4");
+		appdata.buttonpix2 = WinNamedColor("Gray50");
+		appdata.auxpix2 = WinNamedColor("Green");
+		appdata.barpix2 = WinNamedColor("Tan");
+		appdata.parampix2 = WinNamedColor("Plum3");
+		appdata.width = 950;
+		appdata.height = 760;
+		appdata.timeout = 10;
+		appdata.xcfont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
+		appdata.helpfont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
+		appdata.filefont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
+		appdata.textfont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
+		appdata.titlefont = create_font_struct(GetStockObject(DEFAULT_GUI_FONT));
 
-	for (i=0; i<STIPPLES; i++) {
-		STIPPLE[i] = WinCreateBitmapFromData(NULL, STIPDATA[i], 4, 4);
+		for (i=0; i<STIPPLES; i++) {
+			STIPPLE[i] = WinCreateBitmapFromData(NULL, STIPDATA[i], 4, 4);
+		}
+
+		number_colors = 0;
+		colorlist = (colorindex *)malloc(sizeof(colorindex));
+		appcolors = (int *) malloc(NUMBER_OF_COLORS * sizeof(int));
 	}
 
-	number_colors = 0;
-	colorlist = (colorindex *)malloc(sizeof(colorindex));
-	appcolors = (int *) malloc(NUMBER_OF_COLORS * sizeof(int));
-
-
 	clientRect.right = appdata.width+SBARSIZE;
 	clientRect.bottom = appdata.height+SBARSIZE;
 	AdjustWindowRect(&clientRect, WS_TILEDWINDOW, TRUE);
@@ -3299,8 +3362,5 @@ void win32_new_window(Widget w, void *clientData, void *callData)
 	newwin->topinstance = areawin->topinstance;
 
 	areawin = newwin;
-	{
-		BOOL res = ShowWindow(top->handle, SW_SHOW);
-		printf("%d\n", res);
-	}
+	ShowWindow(top->handle, SW_SHOW);
 }
