@@ -66,8 +66,8 @@
 /* Global Variable definitions						*/
 /*----------------------------------------------------------------------*/
 
-short	 popups;      /* total number of popup widgets on the screen */
-int	 pressmode;   /* Whether we are in a press & hold state */
+extern short popups;	/* total number of popup widgets on the screen */
+extern int pressmode;	/* Whether we are in a press & hold state */
 
 xcWidget   top;
 xcWidget   message1, message2, message3, toolbar, overlay = NULL;
@@ -1961,7 +1961,6 @@ XCWindowData *GUI_init(int argc, char *argv[])
 	(XtEventHandler)clientmessagehandler, NULL);
 
    /* Set the area widget width and height, center userspace (0, 0) on screen */
-
    XtSetArg(wargs[0], XtNwidth, &newwin->width);
    XtSetArg(wargs[1], XtNheight, &newwin->height);
    XtGetValues(newwin->area, wargs, 2);
